#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""
    Huawei DLI Python SDK
    The DliClient class of DLI Python SDK
"""

from dli.column import Column
from dli.database import Database
from dli.dli_info import DliInfo
from dli.exception import DliException
from dli.job import SqlJob
from dli.job import SparkBatchJob
from dli.logger import logger
from dli.queue import Queue
from dli.resource import GroupResource, PackageResource, Resources
from dli.rest import GroupResourceInfo
from dli.table import Table, TableSchema
from huaweicloudsdkcore.exceptions import exceptions
from huaweicloudsdkdli.v1.dli_client import DliClient as InnerDliClient
from typing_extensions import deprecated

import time


class DliClient(object):
    MAX_PAGE_SIZE = 100

    def __init__(self, **kwargs):
        """
        Init a DLI client

        :param str auth_mode: 'aksk' mode. Use 'aksk' mode should input ak, sk
        :param str region: The region, for example, cn-north-1
        :param str project_id: On the management console, displayed My Credential page,
                                click Project List to query the project ID,
                                for example, 5a3314075bfa49b9ae360f4ecd333695.
        :param str domain: domain, default: myhuaweicloud.com
        :param str ak: Access Key
        :param str sk: Secret Key
        :param str proxy: proxy address, for example: https://192.168.64.187:3128
        """

        self._sdk_version = '2.0.1'
        self._dli_info = DliInfo(**kwargs)
        from huaweicloudsdkcore.auth.credentials import BasicCredentials
        self._credentials = BasicCredentials(self._dli_info.ak, self._dli_info.sk, self._dli_info.project_id)
        self._inner_client = InnerDliClient.new_builder() \
            .with_credentials(self._credentials) \
            .with_endpoints([self.dli_info.dli]) \
            .build()

    @property
    def sdk_version(self):
        return self._sdk_version

    @property
    def dli_info(self):
        return self._dli_info

    @property
    def inner_client(self):
        return self._inner_client

    @inner_client.setter
    def inner_client(self, inner_client):
        self._inner_client = inner_client

    @property
    def rest_api(self):
        raise DliException("Unsupported")

    @property
    def token_expires_time(self):
        raise DliException("Unsupported")

    @property
    def token(self):
        raise DliException("Unsupported")

    def get_token(self):
        raise DliException("Unsupported")

    def is_token_expired(self):
        raise DliException("Unsupported")

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_queues(self):
        """
        This method is used to list all queues under the project.

        :return: list
                The list of all queues.
        :raises: DliException
        """

        try:
            from huaweicloudsdkdli.v1 import ListQueuesRequest
            resp = self.inner_client.list_queues_invoker(ListQueuesRequest(queue_type="all")) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        queues = []
        for v1_queue in resp.queues:
            # huaweicloudsdkdli.v1.Queue -> Queue
            queues.append(Queue(v1_queue))
        return queues

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_queue(self, queue_name):
        """
        This method is used to get a queue by queue name.

        :param str queue_name: Queue name
        :return: Queue
                If the Queue do not exist, will return None
        :raises: DliException
        """

        try:
            from huaweicloudsdkdli.v1 import ShowQueueRequest
            resp = self.inner_client.show_queue_invoker(ShowQueueRequest(queue_name)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        # ShowQueueResponse -> Queue
        return Queue(resp)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_databases(self):
        """
        This method is used to query the information about all the databases.

        :return: list
                The list of all databases.
        :raises: DliException
        """

        try:
            from huaweicloudsdkdli.v1 import ListDatabasesRequest
            resp = self.inner_client.list_databases_invoker(ListDatabasesRequest()) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        dbs = []
        for v1_db in resp.databases:
            # huaweicloudsdkdli.v1.Database -> Database
            dbs.append(Database(v1_db))
        return dbs

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_database(self, db_name):
        """
        This method is used to get a database by database name.

        :param str db_name: database name
        :return: Database
                If the Database do not exist, will return None
        :raises: DliException
        """

        dbs = self.list_databases()
        for db in dbs:
            if db.name == db_name:
                return db
        return None

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def create_database(self, db_name, description=None):
        """
        This method is used to create a database, make sure database name is not exist.

        :param str db_name: Database name
        :param str description: The description of the database
        :return: Database
        """

        try:
            from huaweicloudsdkdli.v1 import CreateDatabaseRequest, CreateDatabaseRequestBody
            resp = self.inner_client.create_database_invoker(
                CreateDatabaseRequest(CreateDatabaseRequestBody(
                    database_name=db_name, description=description))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self.get_database(db_name)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def create_database_with_eps(self, db_name, enterprise_project_id, description=None):
        """
        This method is used to create a database, make sure database name is not exist, enterprise_project_id is exist.

        :param str db_name: Database name
        :param str enterprise_project_id: EnterpriseProjectId
        :param str description: The description of the database
        :return: Database
        """

        try:
            from huaweicloudsdkdli.v1 import CreateDatabaseRequest, CreateDatabaseRequestBody
            resp = self.inner_client.create_database_invoker(
                CreateDatabaseRequest(CreateDatabaseRequestBody(
                    database_name=db_name, description=description, enterprise_project_id=enterprise_project_id))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self.get_database(db_name)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def delete_database(self, db_name):
        """
        This method is used to delete an empty database. If there are tables
        in the database to be deleted, delete all tables first.

        :param str db_name: Database name
        :return: None
        """

        try:
            from huaweicloudsdkdli.v1 import DeleteDatabaseRequest
            resp = self.inner_client.delete_database_invoker(DeleteDatabaseRequest(database_name=db_name)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_tables(self, db_name, with_detail=False, keyword=None):
        """
        This method is used to query information about tables that meet the filtering
        criteria or all the tables in the specified database.

        :param str db_name: database name
        :param bool with_detail: Whether to get the details information
                of the tables (owner, size, etc.).
        :param str keyword: The keyword of the filter table name.
        :return: list
                The list of all tables in the specified database.
        """

        tables = []
        for i in range(10):
            try:
                from huaweicloudsdkdli.v1 import ListTablesRequest
                resp = self.inner_client.list_tables_invoker(ListTablesRequest(
                    database_name=db_name, current_page=i + 1, page_size=self.MAX_PAGE_SIZE,
                    keyword=keyword, with_detail=with_detail)) \
                    .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
            except exceptions.ClientRequestException as e:
                raise DliException(api_exception=e)
            if not resp.is_success:
                raise DliException(msg=resp.message)

            for v1_tbl in resp.tables:
                # huaweicloudsdkdli.v1.Table -> Table
                tables.append(Table(v1_tbl.table_name, db_name, v1_tbl))
            if resp.table_count < self.MAX_PAGE_SIZE:
                break
        return tables

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_tables_page(self, db_name, page_size, current_page, with_detail=False, keyword=None, table_type=None):
        """
        This method is used to query information about tables that meet the filtering
        criteria or all the tables by page in the specified database.

        :param str db_name: database name
        :param int page_size: the page size to get tables, should be in range 1 to 100
        :param int current_page: current page to get tables
        :param bool with_detail: Whether to get the details information
                 of the tables (owner, size, etc.).
        :param str keyword: The keyword of the filter table name.
        :param str table_type: Filter by table type.(EXTERNAL,MANAGED,VIEW)
        :return: list
                 The list of all tables in the specified database.
        """

        # SDK tableType is difference from rest api, this should convert to the real type in metadata.
        # EXTERNAL -> MANAGED_TABLE, MANAGED -> EXTERNAL_TABLE, VIEW -> VIRTUAL_VIEW
        api_table_type = None
        if table_type:
            if table_type == "EXTERNAL":
                api_table_type = "EXTERNAL_TABLE"
            elif table_type == "MANAGED":
                api_table_type = "MANAGED_TABLE"
            elif table_type == "VIEW":
                api_table_type = "VIRTUAL_VIEW"
            else:
                raise DliException(msg="Invalid table type for list tables: %s" % table_type)

        try:
            from huaweicloudsdkdli.v1 import ListTablesRequest
            resp = self.inner_client.list_tables_invoker(ListTablesRequest(
                database_name=db_name, current_page=current_page, page_size=page_size,
                keyword=keyword, table_type=api_table_type, with_detail=with_detail)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        tables = []
        for v1_tbl in resp.tables:
            # huaweicloudsdkdli.v1.Table -> Table
            tables.append(Table(v1_tbl.table_name, db_name, v1_tbl))
        return resp.table_count, tables

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_table(self, db_name, tbl_name):
        """
        This method is used to get the table in the specified database by table name.

        :param str db_name: database name
        :param str tbl_name: table name
        :return: Table
                 If the table do not exist, will return None
        """

        tables = self.list_tables(db_name, keyword=tbl_name, with_detail=True)
        for table in tables:
            if tbl_name == table.name:
                return table
        return None

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_table_schema(self, db_name, tbl_name):
        """
        This method is used to describe schema information in the specified table.

        :param str db_name: database name
        :param str tbl_name: table name
        :return: TableSchema
        """

        try:
            from huaweicloudsdkdli.v1 import ShowTableRequest
            resp = self.inner_client.show_table_invoker(ShowTableRequest(db_name, tbl_name)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        columns = []
        for v1_col in resp.columns:
            # huaweicloudsdkdli.v1.Column -> Column
            columns.append(Column(name=v1_col.column_name,
                                  data_type=v1_col.type,
                                  desc=v1_col.description,
                                  is_partition_column=v1_col.is_partition_column))
        return TableSchema(tbl_name, columns, [])

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def create_dli_table(self, db_name, schema, description=None):
        """
        This method is used to create DLI table.

        :param str db_name: database name
        :param TableSchema schema: the scheme for the table
        :param str description: table description
        :return: Table
        """

        try:
            from huaweicloudsdkdli.v1 import Column as V1Column, CreateTableRequest, CreateTableRequestBody
            cols = []
            for col in schema.columns:
                cols.append(V1Column(column_name=col.name,
                                     type=col.type,
                                     description=col.description,
                                     is_partition_column=col.is_partition_column))
            resp = self.inner_client.create_table_invoker(CreateTableRequest(
                database_name=db_name, body=CreateTableRequestBody(table_name=schema.table_name,
                                                                   data_location="DLI",
                                                                   description=description,
                                                                   columns=cols))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self.get_table(db_name, schema.table_name)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def create_obs_table(self, db_name, schema, data_path, data_type, description=None, options=None):
        """
        This method is used to create OBS table..

        :param str data_type: Type of data in the new OBS table. Currently, the parameter
                                value can be parquet, ORC, CSV, or JSON.
        :param str data_path: Storage path of data in the new OBS table, which
                                must be a path on OBS and must begin with s3a.
                                For examples: s3a://data/data
        :param str db_name: database name
        :param TableSchema schema: the scheme for the table
        :param str description: table description
        :param dict options: The dict options of the data type, for the detailed information refer to
                            https://support.huaweicloud.com/api-uquery/uquery_02_0100.html
                            For examples:
                            {
                                "with_column_header": true,
                                "delimiter": ",",
                                "quote_char": "\"",
                                "escape_char": "\\",
                                "date_format": "yyyy/MM/dd",
                                "timestamp_format": "yyyy/MM/dd hh:mm:ss"
                            }
        :return: Table
        """

        with_column_header = None
        delimiter = None
        quote_char = None
        escape_char = None
        date_format = None
        timestamp_format = None
        if options:
            with_column_header = options.get('with_column_header')
            delimiter = options.get('delimiter')
            quote_char = options.get('quote_char')
            date_format = options.get('date_format')
            timestamp_format = options.get('timestamp_format')
            escape_char = options.get('escape_char')

        try:
            from huaweicloudsdkdli.v1 import Column as V1Column, CreateTableRequest, CreateTableRequestBody
            cols = []
            for col in schema.columns:
                cols.append(V1Column(column_name=col.name,
                                     type=col.type,
                                     description=col.description,
                                     is_partition_column=col.is_partition_column))
            resp = self.inner_client.create_table_invoker(CreateTableRequest(
                database_name=db_name, body=CreateTableRequestBody(table_name=schema.table_name,
                                                                   data_location='OBS',
                                                                   description=description,
                                                                   columns=cols,
                                                                   data_type=data_type,
                                                                   data_path=data_path,
                                                                   with_column_header=with_column_header,
                                                                   delimiter=delimiter,
                                                                   quote_char=quote_char,
                                                                   escape_char=escape_char,
                                                                   date_format=date_format,
                                                                   timestamp_format=timestamp_format))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return self.get_table(db_name, schema.table_name)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def delete_table(self, db_name, table_name):
        """
        This method is used to delete a specified table.

        :param str db_name: Name of the database where the table to be deleted resides.
        :param str table_name: Name of the table to be deleted.
        :return: None
        """

        try:
            from huaweicloudsdkdli.v1 import DeleteTableRequest
            resp = self.inner_client.delete_table_invoker(
                DeleteTableRequest(database_name=db_name, table_name=table_name)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

    def _submit_sql_job(self, sql, db_name=None, queue_name='default', options=None):
        conf_list = ['dli.sql.sqlasync.enabled=true']
        if options:
            for key, value in options.items():
                conf_list.append(key + '=' + value)

        try:
            from huaweicloudsdkdli.v1 import CreateSqlJobRequest, CreateSqlJobRequestBody
            resp = self.inner_client.create_sql_job_invoker(CreateSqlJobRequest(CreateSqlJobRequestBody(
                currentdb=db_name, sql=sql, queue_name=queue_name, conf=conf_list))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return resp

    def async_execute_sql(self, sql, db_name=None, queue_name='default', options=None):
        """
        Execute SQL job in asynchronous mode.
        This method is used to submit SQL statements to the server cluster.
        The job contains the following types: DDL, DCL, IMPORT, EXPORT, QUERY and INSERT.

        :param str sql: SQL statement
        :param str queue_name: Queue name for the SQL execute, use 'default' queue for default
        :param str db_name: Database name of the table belongs to, if the SQL statement have
                             database name, it can set to None.
        :param dict options: User-defined parameter that applies to the job
                             examples:
                             {'dli.sql.cbo.enabled': True,
                              'dli.sql.syncMode.enabled': True}
        :return: SqlJob
        """

        resp = self._submit_sql_job(sql, db_name, queue_name, options)
        return SqlJob(resp.job_id, resp.job_type, self)

    def execute_sql(self, sql, db_name=None, queue_name='default', options=None, retry_times=1000):
        """
        Execute SQL job
        This method is used to submit SQL statements to the server cluster.
        The job contains the following types: DDL, DCL, IMPORT, EXPORT, QUERY and INSERT.
        This method is a synchronous mode, blocking until SQL execution is completed.

        :param str sql: SQL statement
        :param str queue_name: Queue name for the SQL execute, use 'default' queue for default
        :param str db_name: Database name of the table belongs to, if the SQL statement have
                             database name, it can set to None.
        :param dict options: User-defined parameter that applies to the job
                             examples:
                             {'dli.sql.cbo.enabled': True,
                              'dli.sql.syncMode.enabled': True}
        :return: SqlJob
        """

        resp = self._submit_sql_job(sql, db_name, queue_name, options)
        sql_job = SqlJob(resp.job_id, resp.job_type, self)
        self._cycle_check_sql_job(sql_job, retry_times)
        return sql_job

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def cancel_sql(self, job_id):
        """

        :param str job_id: ID of a job returned after a job is submitted.
        :return:
        """

        try:
            if not job_id:
                raise DliException(msg="No job in progress, cancel job failed")
            from huaweicloudsdkdli.v1 import CancelSqlJobRequest
            resp = self.inner_client.cancel_sql_job_invoker(CancelSqlJobRequest(job_id)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_sql_jobs(self):
        """
        This method is used to list all sql jobs under the project.

        :return: list
                The list of all jobs.
        :raises: DliException
        """

        sql_jobs = []
        job_info_size = 101
        i = 1
        while job_info_size >= 100:
            job_infos = self.list_sql_jobs_page(100, i)
            job_info_size = len(job_infos)
            i += 1
            sql_jobs = sql_jobs + job_infos
        return sql_jobs

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_sql_jobs_page(self, page_size, current_page):
        """
        This method is used to list sql jobs by page under the project.

        :return: list
                The list of all jobs.
        :raises: DliException
        """

        try:
            from huaweicloudsdkdli.v1 import ListSqlJobsRequest
            resp = self.inner_client.list_sql_jobs_invoker(ListSqlJobsRequest(
                current_page=current_page, page_size=page_size)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        sql_jobs = []
        for v1_sql_job in resp.jobs:
            # huaweicloudsdkdli.v1.SqlJob
            sql_jobs.append(v1_sql_job)
        return sql_jobs

    def _cycle_check_sql_job(self, sql_job, retry_times=1000):
        loop_count = 0
        while True:
            if loop_count < 100:
                time.sleep(0.5)
            else:
                time.sleep(3)

            resp_status, resp_msg = sql_job.get_status()
            logger.info("current status is: %s", resp_status)
            if resp_status == 'FAILED':
                message = 'Get job status, Job run failed.\n' \
                          'Job id = {0}\nDLI response: {1}'.format(sql_job.job_id, resp_msg)
                raise DliException(msg=message)
            elif resp_status == 'FINISHED' or resp_status == 'CANCELLED':
                break

            # The sql is running too long times
            if loop_count > retry_times:
                message = 'Job run too long time, Job id = ' + sql_job.job_id
                raise DliException(msg=message)

            loop_count += 1
        return resp_status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def wait_for_finish(self, job_id, retry_times=1000):
        """

        :param str job_id: ID of a job returned after a job is submitted.
        :param int retry_times: The retry interval
        :return: str job_status: The status of this job.
        """

        loop_count = 0
        while True:
            if loop_count < 100:
                time.sleep(0.5)
            else:
                time.sleep(3)

            try:
                from huaweicloudsdkdli.v1 import ShowSqlJobStatusRequest
                resp = self.inner_client.show_sql_job_status_invoker(ShowSqlJobStatusRequest(job_id)) \
                    .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
            except exceptions.ClientRequestException as e:
                raise DliException(api_exception=e)
            if not resp.is_success:
                raise DliException(msg=resp.message)

            if resp.status == 'FAILED':
                message = 'Get job status, Job run failed.\n' \
                          'Job id = {0}\nDLI response: {1}'.format(resp.job_id, resp.message)
                raise DliException(msg=message)
            elif resp.status == 'FINISHED' or resp.status == 'CANCELLED':
                break

            # The sql is running too long times
            if loop_count > retry_times:
                message = 'Job run too long time, Job id = ' + resp.job_id
                raise DliException(msg=message)

            loop_count += 1

        return resp.status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def export_table(self, table_name, db_name, data_path, queue_name='default', compress='none',
                     export_mode='ErrorIfExists'):
        """
        This method is used to export data to a file. You can export data from DLI
        tables to files. Currently, data can be exported only to OBS, and the OBS
        path must be specified to the folder level.
        This method is synchronous, blocking until execution is completed.

        :param str table_name: Name of the table from which data is exported.
        :param str db_name: Name of the database where the table from.
        :param str data_path: Path for storing the exported data (currently,
                            data can be stored on an OBS path starting with s3a only).
        :param str queue_name: Name of the queue that is specified to execute a task.
        :param str compress: Compression mode for exported data. Currently, the
                            compression modes gzip, bzip2, and deflate are supported. If you
                            do not want to compress data, enter none.
        :param str export_mode: Export mode. The parameter value can be ErrorIfExists
                            or Overwrite. If export_mode is not specified, this parameter
                            is set to ErrorIfExists by default.
        :return: tuple (job_id, status)
                    str job_id: ID of a job returned after a job is submitted.
                    str status: The status of this job.
        """

        try:
            from huaweicloudsdkdli.v1 import ExportTableRequest, ExportTableRequestBody
            resp = self.inner_client.export_table_invoker(ExportTableRequest(ExportTableRequestBody(
                data_path=data_path, data_type='csv', compress=compress, queue_name=queue_name,
                database_name=db_name, table_name=table_name, export_mode=export_mode))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        status = self._cycle_check_sql_job(SqlJob(resp.job_id, "UnUsed", self))
        return resp.job_id, status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def import_table(self, table_name, db_name, data_path, data_type, queue_name='default', options=None):
        """
        This method is used to import data from a file to a DLI table. Currently,
        only OBS data can be imported to DLI.
        This method is synchronous, blocking until execution is completed.

        :param str table_name: Name of the table to which data is imported.
        :param str db_name: Name of the database where the table to which data
                            is imported resides.
        :param str data_path: Path to which data is imported. Currently, only data
                            on an OBS path starting with s3a can be imported.
        :param str data_type: Type of the data to be imported. Currently, data types
                            CSV, Parquet, ORC, and JSON are supported.
        :param str queue_name: Name of the queue that is specified to execute a task.
                            If no queue is specified, the default queue is used.
        :param dict options: The dict options of the data type, for the detailed information refer to
                            https://support.huaweicloud.com/api-uquery/uquery_02_0100.html
                            For examples:
                            {
                                "with_column_header": true,
                                "delimiter": ",",
                                "quote_char": "\"",
                                "escape_char": "\\",
                                "date_format": "yyyy/MM/dd",
                                "timestamp_format": "yyyy/MM/dd hh:mm:ss"
                            }
        :return: tuple (job_id, status)
                    str job_id: ID of a job returned after a job is submitted.
                    str status: The status of this job.
        """

        delimiter = None
        quote_char = None
        escape_char = None
        date_format = None
        timestamp_format = None
        with_column_header = None

        if options:
            delimiter = options.get('delimiter')
            quote_char = options.get('quote_char')
            escape_char = options.get('escape_char')
            date_format = options.get('date_format')
            timestamp_format = options.get('timestamp_format')
            with_column_header = options.get('with_column_header')
        from huaweicloudsdkdli.v1 import ImportTableRequestBody
        body = ImportTableRequestBody(data_path=data_path, data_type=data_type,
                                      queue_name=queue_name, database_name=db_name, table_name=table_name,
                                      with_column_header=with_column_header,
                                      delimiter=delimiter, quote_char=quote_char, escape_char=escape_char,
                                      date_format=date_format, timestamp_format=timestamp_format)

        try:
            from huaweicloudsdkdli.v1 import ImportTableRequest
            resp = self.inner_client.import_table_invoker(ImportTableRequest(body)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        status = self._cycle_check_sql_job(SqlJob(resp.job_id, "Unused", self))
        return resp.job_id, status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def submit_spark_batch_job(self, queue_name, batch_job_info):
        """

        Submit spark batch job
        This method is used to submit spark batch job to the server cluster.
        This method is a synchronous mode, blocking until spark batch job is completed.

        :param str queue_name:
        :param SparkJobInfo batch_job_info:
        :return: SparkBatchJob
        """

        body_dict = batch_job_info.get_request_body(queue_name)

        try:
            from huaweicloudsdkdli.v1 import CreateSparkJobRequest, CreateSparkJobRequestBody
            resp = self.inner_client.create_spark_job_invoker(CreateSparkJobRequest(body=CreateSparkJobRequestBody(
                file=body_dict['file'], class_name=body_dict['className'], queue=queue_name,
                args=body_dict['args'], sc_type=body_dict['sc_type'], jars=body_dict['jars'],
                py_files=body_dict['pyFiles'], files=body_dict['files'], conf=body_dict['conf'],
                resources=body_dict['resources'], groups=body_dict['groups'], name=body_dict['name'],
                modules=body_dict['modules'], catalog_name=body_dict['catalog_name'],
                driver_memory=body_dict['driver_memory'], driver_cores=body_dict['driver_cores'],
                executor_memory=body_dict['executor_memory'], executor_cores=body_dict['executor_cores'],
                num_executors=body_dict['num_executors'], obs_bucket=body_dict['obs_bucket'],
                auto_recovery=body_dict['auto_recovery'], max_retry_times=body_dict['max_retry_times'],
                feature=body_dict['feature'], image=body_dict['image']))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)

        return SparkBatchJob(self, resp.id)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def del_spark_batch_job(self, batch_id):
        """

        :param str batch_id: ID of a spark batch job returned after a spark batch job is submitted.
        :return:
        """

        try:
            if not batch_id:
                raise DliException(msg="No spark batch job in progress, cancel spark batch job failed")
            from huaweicloudsdkdli.v1 import CancelSparkJobRequest
            resp = self.inner_client.cancel_spark_job_invoker(CancelSparkJobRequest(batch_id)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        return resp.msg

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_resources(self):
        """
        This method is used to list all resources of the project.

        :return: Resources
        """

        try:
            from huaweicloudsdkdli.v1 import ListJobResourcesRequest
            resp = self.inner_client.list_job_resources_invoker(request=ListJobResourcesRequest()) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)

        pkg_resources = []
        group_resources = []
        for v1_pkg_resource in resp.resources:
            # huaweicloudsdkdli.v1.PackageResource -> PackageResource
            pkg_resources.append(PackageResource(v1_pkg_resource))
        for v1_group_resource in resp.groups:
            group_pkg_resources = []
            for group_pkg_resource in v1_group_resource.details:
                # huaweicloudsdkdli.v1.PackageResource -> PackageResource
                group_pkg_resources.append(PackageResource(group_pkg_resource))
            group_resources.append(GroupResource(GroupResourceInfo(group_name=v1_group_resource.group_name,
                                                                   status=v1_group_resource.status,
                                                                   resources=v1_group_resource.resources,
                                                                   create_time=v1_group_resource.create_time,
                                                                   update_time=v1_group_resource.update_time,
                                                                   details=group_pkg_resources)))
        return Resources(pkg_resources, group_resources)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_package_resource(self, resource_name, group_name=None):
        """
        This method is used to list user-specified package resource of the project

        :param str group_name: name of user defined group
        :param str resource_name: name of package resource
        :return: PackageResource
        """

        try:
            from huaweicloudsdkdli.v1 import ShowJobResourceRequest
            resp = self.inner_client.show_job_resource_invoker(
                ShowJobResourceRequest(resource_name=resource_name, group=group_name)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)

        # ShowJobResourceResponse -> PackageResource
        return PackageResource(resp)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_group_resource(self, group_name):
        """
        This method is used to list user-specified group resource of the project

        :param str group_name: name of user defined group
        :return: GroupResource
        """

        resources = self.list_resources()
        for group_resource in resources.group_resources:
            if group_resource.group_name == group_name:
                return group_resource
        return None

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def upload_resource_async(self, kind, paths, group=None):
        """
        This method is async upload user resources to server.
        This method will return resource names

        :param str kind: jar, py, file
        :param list[str] paths:
        :param str group:
        :return:
        """

        try:
            from huaweicloudsdkdli.v1 import UploadJobResourcesRequest, UploadJobResourcesRequestBody
            resp = self.inner_client.upload_job_resources_invoker(UploadJobResourcesRequest(
                body=UploadJobResourcesRequestBody(kind=kind, paths=paths, group=group, is_async=True))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)

        return resp.resources

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def upload_resource(self, kind, paths, group=None):
        """
        This method is upload user resouces to server.

        :param str kind: jar, py, file
        :param list[str] paths:
        :param str group:
        :return:
        """

        try:
            from huaweicloudsdkdli.v1 import UploadJobResourcesRequest, UploadJobResourcesRequestBody
            self.inner_client.upload_job_resources_invoker(UploadJobResourcesRequest(
                body=UploadJobResourcesRequestBody(kind=kind, paths=paths, group=group, is_async=False))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)

        return

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def delete_resource(self, resource_name, group_name=None):
        """
        This method is used to delete user-specified package resource of the project

        :param str resource_name: name of package resource
        :param str group_name: name of user defined group

        :return:
        """

        try:
            from huaweicloudsdkdli.v1 import DeleteJobResourceRequest
            self.inner_client.delete_job_resource_invoker(
                DeleteJobResourceRequest(resource_name=resource_name, group=group_name)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)

        return

    def set_verify_ssl(self, verify_ssl):
        raise DliException("Unsupported")
