#!/usr/bin/python
# -*- coding: UTF-8 -*-

from dli.column import Column
from dli.exception import DliException
from dli.reader import CsvObsReader, JsonObsReader
from dli.result_set import ResultSet
from dli.row import Row
from dli.table import TableSchema
from huaweicloudsdkcore.exceptions import exceptions
from obs.client import ObsClient
from six import iteritems
from typing_extensions import deprecated

import json


def _handle_data_type(in_type):
    if in_type == 'tinyint' or in_type == 'byte':
        return 'tinyint'
    if in_type == 'smallint' or in_type == 'short':
        return 'smallint'
    if in_type == 'int' or in_type == 'integer':
        return 'int'
    if in_type == 'bigint' or in_type == 'long':
        return 'bigint'
    if in_type == 'string' or in_type.startswith('char') or in_type.startswith('varchar'):
        return 'string'
    if in_type == 'void' or in_type == 'null':
        return 'void'
    return in_type


def _get_schema_from_json(data_dict):
    columns = []
    if data_dict.get('fields') is not None and data_dict.get('fields') != '':
        for col in data_dict.get('fields'):
            col_name = col.get('name')
            col_type = col.get('type')
            if isinstance(col_type, dict):
                col_complex_type = col_type.get('type')
                columns.append(Column(name=col_name, data_type=col_complex_type))
            else:
                columns.append(Column(name=col_name, data_type=_handle_data_type(col_type)))
    return columns


class SqlJob(object):
    def __init__(self, job_id, job_type, client, schema=None, rows=None, status=None, queue_name=None):
        """

        :param str job_id: Job ID.
        :param str job_type: Type of a job, including DDL, DCL, IMPORT, EXPORT, QUERY, and INSERT.
        :param DliClient client: The client of the DLI service.
        :param list schema: Name and type of the job result column.
        :param list rows: Job result set.
        :param str queue_name: Queue name. SQL uses this queue for computing.
        """

        self._job_id = job_id
        self._job_type = job_type
        self._schema = []
        self._rows = []
        self._client = client   # type: DliClient
        self._status = status
        self._queue_name = queue_name
        self._result_path = None
        self._schema_json = None

        if schema is not None:
            for col in schema:
                for col_name, col_type in col.items():
                    self._schema.append(Column(col_name, col_type))

        if rows is not None and schema is not None:
            for row in rows:
                self._rows.append(Row(self._schema, row))

    @property
    def job_id(self):
        return self._job_id

    def set_job_id(self, job_id):
        self._job_id = job_id

    @property
    def job_type(self):
        return self._job_type

    @property
    def job_status(self):
        return self._status

    @property
    def queue_name(self):
        return self._queue_name

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_status(self):
        """
        This method is used to get the job execution status from server.

        :return: str: job status
        """

        try:
            from huaweicloudsdkdli.v1 import ShowSqlJobStatusRequest
            resp = self._client.inner_client.show_sql_job_status_invoker(ShowSqlJobStatusRequest(self.job_id)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self._client.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        self._status = resp.status
        if resp.detail is not None and resp.detail != '':
            if resp.result_path is not None and resp.result_path != '':
                self._result_path = resp.result_path
            self._schema = _get_schema_from_json(json.loads(resp.detail))
        return resp.status, resp.message

    @deprecated("The related API has been brought offline.")
    def get_result_page(self, current_page, page_size=1000, queue_name=None):
        raise DliException("The related API has been brought offline.")

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def preview_result(self):
        try:
            from huaweicloudsdkdli.v1 import PreviewSqlJobResultRequest
            resp = self._client.inner_client.preview_sql_job_result_invoker(
                PreviewSqlJobResultRequest(self.job_id, self.queue_name)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self._client.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        result_schema = []
        for col_info in resp.schema:
            col_dict = json.loads(col_info)
            col_key_list = list(col_dict)
            result_schema.append(Column(name=col_key_list[0], data_type=col_dict.get(col_key_list[0])))
        result_records = []
        for record in resp.rows:
            result_records.append(Row(schema=result_schema, columns=record))
        return ResultSet(job=self, rows=result_records)

    def get_schema(self, queue_name=None):
        """
        This method is used to view the job execution result after a job
        is executed using SQL query statements. Currently, you can only
        query execution results of jobs of the QUERY type.

        :return: list: The list of Columns, have the name and type of the job result column.
        """

        return self._schema

    def get_result(self, page_size=1000, queue_name=None):
        """
        This method is used to view the job execution result after a job
        is executed using SQL query statements. Currently, you can only
        query execution results of jobs of the QUERY type.
        When the query returns more records, it is recommended to use export_result method.

        :return: ResultSet.
        """

        if self._schema is None:
            return None

        if self._result_path is None or self._result_path == '':
            raise DliException("Missing result path")
        if not self._result_path.startswith('obs://'):
            raise DliException("The result path is incorrect")

        result_path = self._result_path[len('obs://'):]
        bucket_name, obj_prefix = result_path.split('/', 1)
        obs_client = ObsClient(server=self._client.dli_info.obs,
                               access_key_id=self._client.dli_info.ak,
                               secret_access_key=self._client.dli_info.sk)
        return CsvObsReader(TableSchema("", self._schema), obs_client, bucket_name, obj_prefix)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def export_result(self, data_path, data_type='csv', queue_name='default', compress='none',
                      export_mode='ErrorIfExists', _async=False, encoding_type='utf-8', retry_times=1000):
        """
        This method is used to export results returned from the query
        using SQL statements to OBS.

        :param str data_path: Path for storing the exported result (currently,
                        data can be stored on an OBS path starting with s3a only). The
                        OBS path cannot contain folders, for example, the path folder
                        in the sample request.
        :param str data_type: Export data type, support csv and json.
        :param str queue_name: Name of the queue that is specified to execute a task.
                        If no queue is specified, the default queue is used.
        :param str compress: Compression format of exported data. Currently, gzip,
                        bzip2, and deflate are supported. The default value is
                        none, indicating that data is not compressed.
        :param str export_mode: Export mode. The parameter value can be
                        ErrorIfExists or Overwrite. If export_mode is not
                        specified, this parameter is set to ErrorIfExists by default.
        :param bool _async: If running export job in async. The default value is False.
                        If true then just return job id.
        :param encoding_type: Export file encoding type.
        :param int retry_times: The retry interval
        :return: str Status of a job, including LAUNCHING, RUNNING,
                        FINISHED, FAILED, and CANCELLED.
        """

        try:
            from huaweicloudsdkdli.v1 import ExportSqlJobResultRequest, ExportSqlJobResultRequestBody
            resp = self._client.inner_client.export_sql_job_result_invoker(
                ExportSqlJobResultRequest(job_id=self.job_id, body=ExportSqlJobResultRequestBody(
                    queue_name=queue_name, data_path=data_path, data_type=data_type, compress=compress,
                    export_mode=export_mode, encoding_type=encoding_type))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self._client.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        if _async:
            return resp.job_id
        return self._client.wait_for_finish(resp.job_id, retry_times)

    def export_job_result(self, data_path, queue_name, retry_times=1000):
        """
        This method is used to export results returned from the query
        using SQL statements to OBS.

        :param str data_path: Path for storing the exported result (currently,
                        data can be stored on an OBS path starting with obs only). The
                        OBS path cannot contain folders, for example, the path folder
                        in the sample request.
        :param str queue_name: Name of the queue that is specified to execute a task.
                        If no queue is specified, the default queue is used.
        :param int retry_times: The retry interval
        :return: Reader.
        """

        if self._schema is None:
            return None

        if not data_path.startswith("obs://"):
            raise DliException("Invalid data path")
        self.export_result(data_path=data_path, data_type="json", queue_name=queue_name, retry_times=retry_times)
        bucket_name, obj_prefix = data_path[6:].split('/', 1)
        obs_client = ObsClient(server=self._client.dli_info.obs,
                               access_key_id=self._client.dli_info.ak,
                               secret_access_key=self._client.dli_info.sk)
        return JsonObsReader(TableSchema("", self._schema), obs_client, bucket_name, obj_prefix)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def cancel_job(self):
        """
        This method is used to cancel the job on server.

        :return: bool: is success
        """

        self._client.cancel_sql(self.job_id)


class SparkJobInfo(object):
    def __init__(self, app_file, **kwargs):
        """
        Init the SparkJobInfo instance

        :param app_file:
        :param kwargs:
        """

        all_params = ['app_file', 'app_name', 'class_name', 'jars', 'py_files', 'files',
                      'confs', 'resources', 'groups', 'modules', 'sc_type', 'args', 'catalog_name',
                      'driver_memory', 'driver_cores', 'executor_memory', 'executor_cores', 'num_executors',
                      'obs_bucket', 'auto_recovery', 'max_retry_times', 'feature', 'image']

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'" % key)
            params[key] = val

        del params['kwargs']

        self._name = params.get('app_name')
        self._file = params.get('app_file')
        self._class = params.get('class_name')
        self._jars = params.get('jars')
        self._py_files = params.get('py_files')
        self._files = params.get('files')
        self._confs = params.get('confs')
        self._resources = params.get('resources')
        self._groups = params.get('groups')
        self._sc_type = params.get('sc_types')
        self._args = params.get('args')
        self._modules = params.get('modules')
        self._catalogName = params.get('catalog_name')
        self._driverMemory = params.get('driver_memory')
        self._driverCores = params.get('driver_cores')
        self._executorMemory = params.get('executor_memory')
        self._executorCores = params.get('executor_cores')
        self._numExecutors = params.get('num_executors')
        self._obsBucket = params.get('obs_bucket')
        self._autoRecovery = params.get('auto_recovery')
        self._maxRetryTimes = params.get('max_retry_times')
        self._feature = params.get('feature')
        self._image = params.get('image')

    def get_request_body(self, queue_name):

        # build a json object of the api request body for submit spark job
        rest_request_body = {
            'file': self._file,
            'className': self._class,
            'queue': queue_name,
            'args': self._args,
            'sc_type': self._sc_type,
            'jars': self._jars,
            'pyFiles': self._py_files,
            'files': self._files,
            'conf': self._confs,
            'resources': self._resources,
            'groups': self._groups,
            'name': self._name,
            'modules': self._modules,
            'catalog_name': self._catalogName,
            'driver_memory': self._driverMemory,
            'driver_cores': self._driverCores,
            'executor_memory': self._executorMemory,
            'executor_cores': self._executorCores,
            'num_executors': self._numExecutors,
            'obs_bucket': self._obsBucket,
            'auto_recovery': self._autoRecovery,
            'max_retry_times': self._maxRetryTimes,
            'feature': self._feature,
            'image': self._image
        }

        return rest_request_body


class SparkBatchJob(object):
    def __init__(self, client, job_id):
        """
        Init the SparkBatchJob instance

        :param str job_id: The job_id return form the server
        :param DliClient client: The object of the DLIClient
        """

        self._job_id = job_id
        self._client = client   # type: DliClient
        self._status = None

    @property
    def job_id(self):
        return self._job_id

    @property
    def status(self):
        return self._status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_job_status(self):
        """
        Get Spark batch job status from dli server

        :return: str Status include 'starting','running','dead','success','recovering'
        """

        try:
            from huaweicloudsdkdli.v1 import ShowSparkJobStatusRequest
            resp = self._client.inner_client.show_spark_job_status_invoker(ShowSparkJobStatusRequest(self.job_id)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self._client.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)

        self._status = resp.state
        return resp.state

    @deprecated("The related API has been brought offline.")
    def get_driver_log(self, size):
        raise DliException("The related API has been brought offline.")
