#!/usr/bin/python
# -*- coding: UTF-8 -*-

from huaweicloudsdkcore.auth.credentials import BasicCredentials
from huaweicloudsdkcore.exceptions import exceptions
from huaweicloudsdkdli.v1 import BatchRunFlinkJobsRequest
from huaweicloudsdkdli.v1 import BatchRunFlinkJobsRequestBody
from huaweicloudsdkdli.v1 import CreateFlinkJarJobRequestBody
from huaweicloudsdkdli.v1 import CreateFlinkJarJobRequest
from huaweicloudsdkdli.v1 import DliClient
from huaweicloudsdkdli.v1 import ShowFlinkJobRequest
from huaweicloudsdkdli.v1.region.dli_region import DliRegion
import os
import time
import logging

# 配置日志
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def main():
    """
    基础认证信息：
    ak: 华为云账号Access Key
    sk: 华为云账号Secret Access Key
    认证用的ak和sk直接写到代码中有很大的安全风险，建议在配置文件或者环境变量中密文存放，使用时解密，确保安全；
    本示例以ak和sk保存在环境变量中来实现身份验证为例，运行本示例前请先在本地环境中设置环境变量HUAWEICLOUD_SDK_AK和HUAWEICLOUD_SDK_SK。

    通过以下方式可获取AK/SK，项目ID及对应的region信息。
    登录管理控制台。
    鼠标指向界面右上角的登录用户名，在下拉列表中单击“我的凭证”。
    在左侧导航栏中选择“访问密钥”，单击“新增访问密钥”。根据提示输入对应信息，单击“确定”。
    在弹出的提示页面单击“立即下载”。下载成功后，打开凭证文件，获取AK/SK信息。
    左侧导航栏单击“API凭证”，在“项目列表”中获取“项目ID”即为project_id值，对应的“项目”即为region的值。
    """
    your_access_key = os.getenv("HUAWEICLOUD_SDK_AK")
    your_secret_key = os.getenv("HUAWEICLOUD_SDK_SK")
    project_id = "your_project_id"
    region_name = "region_name"  # 比如 "cn-east-3"

    credentials = BasicCredentials(your_access_key, your_secret_key, project_id)
    dli_client = DliClient.new_builder() \
        .with_credentials(credentials) \
        .with_region(DliRegion.value_of(region_name)) \
        .build()

    try:
        # 步骤一：创建Flink作业。作业状态将变成 “草稿”
        job_id = create_flink_jar_job(dli_client, "your_queue_name")
        logger.info("job_id: %d", job_id)

        # 步骤二：运行作业。作业状态将从 “草稿” 变成 “提交中”
        resps = batch_run_flink_jar_jobs(dli_client, [job_id])
        logger.info("Response: %s", resps)

        # 步骤三：查询作业状态。如果您想在当前线程等待作业进入“运行中”状态，可循环检查状态，直到作业进入“运行中”状态。
        check_running(dli_client, job_id)

    except exceptions.ClientRequestException as e:
        # 请根据业务实际情况处理异常信息，此处仅作样例。
        logger.error("Failed to execute job:", e)


def create_flink_jar_job(client, queue_name):
    """
    创建 Flink jar 作业。作业状态将变成 “草稿”
    关键SDK API：huaweicloudsdkdli.v1.dli_client.DliClient.create_flink_jar_job，详见 https://console.huaweicloud.com/apiexplorer/#/openapi/DLI/doc?api=CreateFlinkJarJob

    :param huaweicloudsdkdli.v1.dli_client.DliClient client: DLI Client
    :param str queue_name: Queue name for the job to execute
    :return: int
    """
    request_body = CreateFlinkJarJobRequestBody(
        name="your_job_name",  # 作业名称，名字必须唯一,比如 flink_jar_job_demo。长度限制：1-57个字符。
        desc="your flink job's description",  # 用户自定义描述。长度限制：0-512个字符。
        queue_name=queue_name,  # 通用队列名称。队列名称。长度限制：0-128个字符。
        feature="basic",  # 作业特性。表示用户作业使用的Flink镜像类型。basic：表示使用DLI提供的基础Flink镜像。
        flink_version="1.12",  # Flink版本。当用户设置“feature”为“basic”时，该参数生效。用户可通过与“feature”参数配合使用，指定作业运行使用的DLI基础Flink镜像的版本
        log_enabled=True,  # 是否开启作业日志。
        obs_bucket="your_obs_bucket_name",  # 当“log_enabled”为“true”时, 用户授权保存作业日志的OBS桶名。
        entrypoint="obs://your_obs_bucket_name/your/flink/job.jar",  # 用户已上传到OBS的程序包，用户自定义作业主类所在的jar包。
        main_class="your_class_fullname"  # 作业入口类,比如:org.apache.flink.examples.WordCount
    )
    request = CreateFlinkJarJobRequest(body=request_body)
    response = client.create_flink_jar_job(request)
    return response.job.job_id


def batch_run_flink_jar_jobs(client, job_ids):
    """
    运行作业。作业状态将从 “草稿” 变成 “提交中”
    关键SDK API：huaweicloudsdkdli.v1.dli_client.DliClient.batch_run_flink_jobs，详见 https://console.huaweicloud.com/apiexplorer/#/openapi/DLI/doc?api=BatchRunFlinkJobs

    :param huaweicloudsdkdli.v1.dli_client.DliClient client: DLI Client.
    :param list[int] job_ids: The job ids for running.
    :return: The body of this BatchRunFlinkJobsResponse.
    :rtype: list[:class:`huaweicloudsdkdli.v1.FlinkSuccessResponse`]
    """
    request_body = BatchRunFlinkJobsRequestBody(job_ids=job_ids)
    request = BatchRunFlinkJobsRequest(body=request_body)
    response = client.batch_run_flink_jobs(request)
    return response.body


def check_running(client, job_id):
    """
    如果您想在当前线程等待作业进入“运行中”状态，可执行此方法，循环检查状态，直到作业进入“运行中”状态。
    关键SDK API：huaweicloudsdkdli.v1.dli_client.DliClient.show_flink_job，详见 https://console.huaweicloud.com/apiexplorer/#/openapi/DLI/sdk?api=ShowFlinkJob

    :param huaweicloudsdkdli.v1.dli_client.DliClient client: DLI Client.
    :param int job_id: The job id for getting status.
    :return:
    """
    while True:
        try:
            request = ShowFlinkJobRequest(job_id=job_id)
            response = client.show_flink_job(request)
        except exceptions.ClientRequestException as e:
            raise Exception(f"Failed to get Flink jar job status by id: {job_id}") from e

        status = response.job_detail.status
        logger.info("FlinkJarJob id %d status: %s", job_id, status)

        if status == "job_running":
            return
        if status in ["job_submit_fail", "job_running_exception"]:
            raise Exception(f"Run Flink jar job failed: {response}")

        time.sleep(1)


if __name__ == "__main__":
    main()
